/*---------------------------------------------------------------------------*\
**$Author: saulius $
**$Date: 2018-03-07 10:02:59 +0200 (Wed, 07 Mar 2018) $
**$Revision: 140 $
**$URL: svn+ssh://www.crystallography.net/home/coder/svn-repositories/smiles-scripts/trunk/src/cdkdepict.java $
\*---------------------------------------------------------------------------*/

// This option sets output format to EPS.

class EPSOutputOption extends OutputOption {

    EPSOutputOption( String short_name, String long_name,
                     OptionValue opt_value )
    {
        super( short_name, long_name, opt_value, "EPS" );
    }
}

