public class CDKSdf2InChIHelp extends VersionOption {

    CDKSdf2InChIHelp( String short_name, String long_name, OptionType opt_type )
    {
        super( short_name, long_name, opt_type );
    }

    int proc( String args[], int i ) {
        System.out.print
            ( "Convert SDF (MOL) files to InChI or InChIKey strings.\n\n" +

              "USAGE\n" +
              "    " + progName + " < input.sdf\n" +
              "    " + progName + " inputs*.sdf\n" +
              "OPTIONS:\n" +
              "    -2, --mdl2000-input  read MDL2000 format files (default)\n" +
              "    -3, --mdl3000-input  read MDL3000 format files\n\n" +

              "    -h, --suppress-hydrogens\n" +
              "        suppress explicit hydrogens (default)\n\n" +

              "    -h-, --dont-suppress-hydrogens\n" +
              "        leave explicit hydrogens\n\n" +

              "    --version  Print program version and exit\n" +
              "    --help     Print short help message (this message) and exit\n"
        );
        System.exit(0);
        return i;
    }

}
