// Print out help message and exit; to be passed to the SOptions package.
class CDKRechargeHelp extends VersionOption {

    CDKRechargeHelp( String short_name, String long_name, OptionType opt_type )
    {
        super( short_name, long_name, opt_type );
    }

    int proc( String args[], int i ) {
        System.out.print
            ( "Redistribute charges on haloanions so that charge is placed\n" +
              "on central atom, not on periferal halogen atoms; e.g.\n" +
              "'[B](F)(F)(F)[F-]' is transformed to '[B-](F)(F)(F)F'.\n\n" +

              "USAGE\n" +
              "    " + progName + " < input.smi\n" +
              "    " + progName + " inputs*.smi\n" +
              "OPTIONS:\n" +
              "    -k,  --kekulise\n" +
              "        Attempt to \"kekulise\" aromatic smiles (default)\n\n" +
              "    -k-, --dont-kekulise, --do-not-kekulise\n" +
              "        Do not \"kekulise\" aromatic smiles, " +
              "leave input as it is\n\n" +
              "    --smiles c1ccccc1\n" +
              "        Specify SMILES string to redistribute charges " +
              "instead of reading input files\n" +
              "\n" +
              "    --version  Print program version and exit\n" +
              "    --help     Print short help message (this message) and exit\n"
        );
        System.exit(0);
        return i;
    }
}
