#!/usr/bin/perl

use strict;
use warnings;

use SmilesScripts::OPSIN;

my $opsin = SmilesScripts::OPSIN->new;

local $\ = "\n";
for ('benzene', 'milk', 'picene') {
    my $SMILES = $opsin->IUPAC_to_SMILES( $_ );
    print defined $SMILES ? $SMILES : 'undef';
}
