#!/bin/sh

sudo apt-get install -y \
    default-jdk \
    default-jre \
    gcc \
    libalgorithm-combinatorics-perl \
    libcgi-pm-perl \
    libchemistry-elements-perl \
    libclone-perl \
    libgetopt-long-descriptive-perl \
    libgraph-perl \
    libipc-run-perl \
    libmath-matrixreal-perl \
    libmodule-build-perl \
    libnauty2-dev \
    libopsin-java \
    libparse-yapp-perl \
    liburl-encode-perl \
    libxml-libxml-perl \
    openbabel \
    make \
;
