#!/bin/bash

echo -e "XXXXXXX\tClC1=C(\C=N\C2C(CCCC2)/N=C/C2=C(C=CC=C2)Cl)C=CC=C1\tClc1c(/C=N/[C@H]2[C@H](/N=C/c3c(Cl)cccc3)CCCC2)cccc1.Clc1c(/C=N/[C@@H]2[C@@H](/N=C/c3c(Cl)cccc3)CCCC2)cccc1" \
    | smi_isomorphism
