#!/bin/bash

echo -e "XXXXXXX\tC1(=CC=CC=C1)C1=NN=C(O1)S(C=1C(=C(C(=C(C1C)S(=C)C=1OC(=NN1)C1=CC=CC=C1)C)S(=C)C=1OC(=NN1)C1=CC=CC=C1)C)=C\tS(Cc1c(c(c(c(c1C)CSc1oc(nn1)c1ccccc1)C)CSc1oc(nn1)c1ccccc1)C)c1oc(nn1)c1ccccc1" \
    | smi_isomorphism
