/*---------------------------------------------------------------------------*\
**$Author: saulius $
**$Date: 2018-03-07 10:02:59 +0200 (Wed, 07 Mar 2018) $
**$Revision: 140 $
**$URL: svn+ssh://www.crystallography.net/home/coder/svn-repositories/smiles-scripts/trunk/src/cdkdepict.java $
\*---------------------------------------------------------------------------*/

// An Option class to specify output format.

class OutputOption extends Option {

    private final String fmt;

    OutputOption( String short_name, String long_name,
                  OptionValue opt_value, String format )
    {
        super( short_name, long_name, OptionType.OT_FUNCTION, opt_value );
        this.fmt = format;
    }

    public String getFormat() {
        return this.fmt;
    }

    // The method that will be called when an option is activated on
    // the command line:
    int proc( String args[], int i ) {
        this.value.s = getFormat();
        return i;
    }
}

