#!/bin/sh
##
# Test the way filenames with Unicode characters are printed in error messages.
##

SCRIPT_NAME=smi_parse
INPUT_FILE=tests/inputs/${SCRIPT_NAME}/žąsies-šypsena.smi

${SCRIPT_NAME} ${INPUT_FILE}
