public class CDKCif2SmilesHelp extends VersionOption {

    CDKCif2SmilesHelp( String short_name, String long_name, OptionType opt_type )
    {
        super( short_name, long_name, opt_type );
    }

    int proc( String args[], int i ) {
        System.out.print
            ( "Covert CIFs (Crystallographic Information Files) to SMILES.\n\n" +

              "USAGE\n" +
              "    " + progName + " < input.cif\n" +
              "    " + progName + " inputs*.cif\n\n" +
              "OPTIONS:\n" +
              "    --version  Print program version and exit\n" +
              "    --help     Print short help message (this message) and exit\n"
            );
        System.exit(0);
        return i;
    }

}
