#!/bin/bash

echo 'C/C=C\CCCCC/C=C\C' | smi_canonicalise
echo 'C/C=C\CCCCC/C=C\C' | smi_canonicalise | smi_canonicalise
echo 'C/C=C\CCCCC\C=C/C' | smi_canonicalise
echo 'C/C=C\CCCCC\C=C/C' | smi_canonicalise | smi_canonicalise
